import { Component, OnInit, OnDestroy } from '@angular/core';
import { ActivatedRoute, ParamMap } from '@angular/router';
import { Subscription } from 'rxjs';

import { ContactsService, Contact } from '../contacts.service';

@Component({
  selector: 'app-contacts-wrapper',
  template: `
    <div *ngIf="errorMessage" class="error">{{errorMessage}}</div>
    <div *ngIf="currentContactsListId; else elseBlock">
      <div [ngSwitch]="display">
        <div *ngSwitchCase="'list'">
          <div class="btn-row">
            <button (click)="onAdd()">Hinzufügen</button>
          </div>
          <app-contacts
            [contacts]="contacts"
            (edit)="onEdit($event)"
            (delete)="onDelete($event)"
          ></app-contacts>
        </div>
        <app-add-contact *ngSwitchCase="'add'" (done)="onDone()" [currentContactsListId]="currentContactsListId"></app-add-contact>
        <app-edit-contact *ngSwitchCase="'edit'" (done)="onDone()"
          [currentContactsListId]="currentContactsListId" [contactId]="currentContactId"></app-edit-contact>
      </div>
    </div>
    <ng-template #elseBlock>
      Klicken Sie auf eine Kontaktliste
    </ng-template>
  `,
  styles: ['app-contacts { display: block; margin-top: 10px; }']
})
export class ContactsWrapperComponent implements OnInit, OnDestroy {
  contacts: Array<Contact> = [];
  display = 'list';
  currentContactId: string;
  errorMessage = '';

  currentContactsListId: string;

  getSubscription: Subscription;
  doneSubscription: Subscription;
  deleteSubscription: Subscription;
  getAfterDeleteSubscription: Subscription;
  routeSubscription: Subscription;

  constructor(
    private contactsService: ContactsService,
    private route: ActivatedRoute
  ) {}

  ngOnInit() {
    this.routeSubscription = this.route.paramMap.subscribe((map: ParamMap) => {
      this.currentContactsListId = map.get('listId');
      this.getSubscription = this.getContacts();
    });
  }

  ngOnDestroy() {
    this.getSubscription.unsubscribe();

    if (this.doneSubscription) {
      this.doneSubscription.unsubscribe();
    }

    if (this.deleteSubscription) {
      this.deleteSubscription.unsubscribe();
    }

    if (this.getAfterDeleteSubscription) {
      this.getAfterDeleteSubscription.unsubscribe();
    }

    if (this.routeSubscription) {
      this.routeSubscription.unsubscribe();
    }
  }

  getContacts() {
    return this.contactsService
      .getContacts(this.currentContactsListId)
      .subscribe(
        contacts => {
          this.contacts = contacts;
        },
        (err: Error) => {
          this.errorMessage = err.message;
        }
      );
  }

  onAdd() {
    this.display = 'add';
  }

  onDone() {
    this.display = 'list';
    this.doneSubscription = this.getContacts();
  }

  onEdit(id: string) {
    this.display = 'edit';
    this.currentContactId = id;
  }

  onDelete(id: string) {
    this.deleteSubscription = this.contactsService
      .deleteContact(this.currentContactsListId, id)
      .subscribe(
        () => {
          this.getAfterDeleteSubscription = this.getContacts();
        },
        (err: Error) => {
          this.errorMessage = err.message;
        }
      );
  }
}
