import { ComponentFixture, TestBed } from '@angular/core/testing';
import { Router } from '@angular/router';
import { NO_ERRORS_SCHEMA, Component } from '@angular/core';
import { of, throwError } from 'rxjs';
import { By } from '@angular/platform-browser';

import { ContactsListsWrapperComponent } from './contacts-lists-wrapper.component';
import { ContactsListsService } from '../contacts-lists.service';

@Component({
  selector: 'app-contacts-lists',
  template: ''
})
class ContactsListsStubComponent {}

@Component({
  selector: 'app-add-contacts-list',
  template: ''
})
class AddContactsListStubComponent {}

@Component({
  selector: 'app-edit-contacts-list',
  template: ''
})
class EditContactsListStubComponent {}

describe('ContactsListsWrapperComponent', () => {
  let fixture: ComponentFixture<ContactsListsWrapperComponent>;
  let contactsListsWrapperComponent: ContactsListsWrapperComponent;
  let contactsListsService: jasmine.SpyObj<ContactsListsService>;
  let router: Router;

  beforeEach(() => {
    TestBed.configureTestingModule({
      declarations: [
        ContactsListsWrapperComponent,
        ContactsListsStubComponent,
        AddContactsListStubComponent,
        EditContactsListStubComponent
      ],
      providers: [
        {
          provide: ContactsListsService,
          useValue: jasmine.createSpyObj('ContactsListsService', [
            'getContactsLists',
            'deleteContactsList'
          ])
        },
        {
          provide: Router,
          useValue: jasmine.createSpyObj('Router', ['navigate'])
        }
      ],
      schemas: [NO_ERRORS_SCHEMA]
    });

    contactsListsService = TestBed.get(ContactsListsService);
    router = TestBed.get(Router);

    fixture = TestBed.createComponent(ContactsListsWrapperComponent);
    contactsListsWrapperComponent = fixture.componentInstance;
  });

  describe('getContactsLists', () => {
    it('should call the getContactsLists method of the service', () => {
      contactsListsService.getContactsLists.and.returnValue(of([]));

      contactsListsWrapperComponent.getContactsLists();

      expect(contactsListsService.getContactsLists).toHaveBeenCalled();
    });

    it('should set the value of contactsLists', () => {
      const serverReturn = [{ name: 'foo', id: '123' }];
      contactsListsService.getContactsLists.and.returnValue(of(serverReturn));

      contactsListsWrapperComponent.getContactsLists();

      expect(contactsListsWrapperComponent.contactsLists).toEqual(serverReturn);
    });

    it('should show an error message if getting the data failed', () => {
      const msg = 'SOME_ERROR';

      contactsListsService.getContactsLists.and.returnValue(
        throwError(new Error(msg))
      );

      contactsListsWrapperComponent.getContactsLists();

      fixture.detectChanges();

      const elem = fixture.debugElement.query(By.css('.error'));

      expect(elem.nativeElement.textContent).toContain(msg);
    });
  });

  it('should get the contacts lists onInit', () => {
    const getContactsListsSpy = spyOn(
      contactsListsWrapperComponent,
      'getContactsLists'
    );

    contactsListsWrapperComponent.ngOnInit();

    expect(getContactsListsSpy).toHaveBeenCalled();
  });

  it('should show the ContactsListsComponent when initialized', () => {
    contactsListsService.getContactsLists.and.returnValue(of([]));

    fixture.detectChanges();

    expect(contactsListsWrapperComponent.display).toBe('list');

    const elem = fixture.debugElement.query(
      By.directive(ContactsListsStubComponent)
    );

    expect(elem).not.toBeNull();
  });

  it('should navigate to the correct contacts onSelect', () => {
    const listId = '123';
    contactsListsWrapperComponent.onSelect(listId);

    expect(router.navigate).toHaveBeenCalledWith(['/', listId, 'contacts']);
  });

  it('should display AddContactsList onAdd', () => {
    contactsListsService.getContactsLists.and.returnValue(of([]));

    contactsListsWrapperComponent.onAdd();

    fixture.detectChanges();

    expect(contactsListsWrapperComponent.display).toBe('add');

    const elem = fixture.debugElement.query(
      By.directive(AddContactsListStubComponent)
    );

    expect(elem).not.toBeNull();
  });

  it('should display ContactsLists and get contacts lists onDone', () => {
    const getContactsListsSpy = spyOn(
      contactsListsWrapperComponent,
      'getContactsLists'
    );

    contactsListsWrapperComponent.onDone();

    fixture.detectChanges();

    expect(getContactsListsSpy).toHaveBeenCalled();
    expect(contactsListsWrapperComponent.display).toBe('list');

    const elem = fixture.debugElement.query(
      By.directive(ContactsListsStubComponent)
    );

    expect(elem).not.toBeNull();
  });

  it('should display EditContactsList onEdit', () => {
    contactsListsService.getContactsLists.and.returnValue(of([]));
    const listId = '123';
    contactsListsWrapperComponent.onEdit(listId);

    fixture.detectChanges();

    expect(contactsListsWrapperComponent.display).toBe('edit');
    expect(contactsListsWrapperComponent.currentContactsListId).toBe(listId);

    const elem = fixture.debugElement.query(
      By.directive(EditContactsListStubComponent)
    );

    expect(elem).not.toBeNull();
  });

  describe('onDelete', () => {
    it('should call deleteContactsList with the correct id', () => {
      contactsListsService.getContactsLists.and.returnValue(of([]));
      contactsListsService.deleteContactsList.and.returnValue(of(''));

      const listId = '123';
      contactsListsWrapperComponent.onDelete(listId);

      expect(contactsListsService.deleteContactsList).toHaveBeenCalledWith(
        listId
      );
    });

    it('should get the contacts lists', () => {
      contactsListsService.deleteContactsList.and.returnValue(of(''));

      const getContactsListsSpy = spyOn(
        contactsListsWrapperComponent,
        'getContactsLists'
      );

      const listId = '123';
      contactsListsWrapperComponent.onDelete(listId);

      expect(getContactsListsSpy).toHaveBeenCalled();
    });

    it('should show an error message if the deletion failed', () => {
      contactsListsService.getContactsLists.and.returnValue(of([]));

      const msg = 'SOME_ERROR';

      contactsListsService.deleteContactsList.and.returnValue(
        throwError(new Error(msg))
      );

      const listId = '123';
      contactsListsWrapperComponent.onDelete(listId);

      fixture.detectChanges();

      const elem = fixture.debugElement.query(By.css('.error'));

      expect(elem.nativeElement.textContent).toContain(msg);
    });
  });
});
