import { ComponentFixture, TestBed } from '@angular/core/testing';
import { NO_ERRORS_SCHEMA } from '@angular/core';
import { By } from '@angular/platform-browser';
import { of, throwError } from 'rxjs';

import { EditContactsListComponent } from './edit-contacts-list.component';
import { ContactsListsService } from '../contacts-lists.service';

describe('EditContactsListComponent', () => {
  let fixture: ComponentFixture<EditContactsListComponent>;
  let editContactsListComponent: EditContactsListComponent;
  let contactsListsService: jasmine.SpyObj<ContactsListsService>;

  beforeEach(() => {
    TestBed.configureTestingModule({
      declarations: [EditContactsListComponent],
      providers: [
        {
          provide: ContactsListsService,
          useValue: jasmine.createSpyObj('ContactsService', [
            'updateContactsList',
            'getContactsList'
          ])
        }
      ],
      schemas: [NO_ERRORS_SCHEMA]
    });

    contactsListsService = TestBed.get(ContactsListsService);

    fixture = TestBed.createComponent(EditContactsListComponent);
    editContactsListComponent = fixture.componentInstance;
  });

  it('should call getContactsList on init', () => {
    contactsListsService.getContactsList.and.returnValue(of({}));
    const listId = '123';
    editContactsListComponent.contactsListId = listId;

    editContactsListComponent.ngOnInit();

    expect(contactsListsService.getContactsList).toHaveBeenCalledWith(listId);
  });

  it('should set the contactsList property if the getContactsList call was successful', () => {
    const listId = '123';
    const contactsList = {
      id: listId,
      name: 'foobar'
    };
    contactsListsService.getContactsList.and.returnValue(of(contactsList));

    editContactsListComponent.contactsListId = listId;

    editContactsListComponent.ngOnInit();

    expect(editContactsListComponent.contactsList).toEqual(contactsList);
  });

  it('should show an error message if the getContactsList call failed', () => {
    const msg = 'SOME_ERROR';
    contactsListsService.getContactsList.and.returnValue(
      throwError(new Error(msg))
    );
    editContactsListComponent.ngOnInit();

    fixture.detectChanges();

    const elem = fixture.debugElement.query(By.css('.error'));

    expect(elem.nativeElement.textContent).toContain(msg);
  });

  it('should call updateContactsList on submit', () => {
    contactsListsService.updateContactsList.and.returnValue(of(''));
    contactsListsService.getContactsList.and.returnValue(of({}));

    const contactsListId = '123';
    editContactsListComponent.contactsListId = contactsListId;

    editContactsListComponent.ngOnInit();

    const data = { name: 'foobar' };

    editContactsListComponent.onSubmit(data);

    expect(contactsListsService.updateContactsList).toHaveBeenCalledWith(
      contactsListId,
      data
    );
  });

  it('should emit done if the updateContactsList call was successful', () => {
    contactsListsService.updateContactsList.and.returnValue(of(''));
    contactsListsService.getContactsList.and.returnValue(of({}));

    editContactsListComponent.contactsListId = '123';

    editContactsListComponent.ngOnInit();

    const data = { name: 'abc' };

    const doneSpy = spyOn(editContactsListComponent.done, 'emit');

    editContactsListComponent.onSubmit(data);

    expect(doneSpy).toHaveBeenCalled();
  });

  it('should show an error message if the updateContactsList call failed', () => {
    const msg = 'SOME_ERROR';
    contactsListsService.updateContactsList.and.returnValue(
      throwError(new Error(msg))
    );
    contactsListsService.getContactsList.and.returnValue(of({}));
    editContactsListComponent.ngOnInit();

    const data = { name: 'abc' };

    editContactsListComponent.onSubmit(data);

    fixture.detectChanges();

    const elem = fixture.debugElement.query(By.css('.error'));

    expect(elem.nativeElement.textContent).toContain(msg);
  });

  it('should unsubscribe the update subscription on destroy', () => {
    const data = { name: 'abc' };
    contactsListsService.getContactsList.and.returnValue(of({}));
    contactsListsService.updateContactsList.and.returnValue(of(''));
    editContactsListComponent.ngOnInit();

    editContactsListComponent.onSubmit(data);

    const unsubscribeSpy = spyOn(
      editContactsListComponent.updateSubscription,
      'unsubscribe'
    ).and.callThrough();

    editContactsListComponent.ngOnDestroy();

    expect(unsubscribeSpy).toHaveBeenCalled();
  });

  it('should unsubscribe the get subscription on destroy', () => {
    contactsListsService.getContactsList.and.returnValue(of({}));
    editContactsListComponent.ngOnInit();

    const unsubscribeSpy = spyOn(
      editContactsListComponent.getSubscription,
      'unsubscribe'
    ).and.callThrough();

    editContactsListComponent.ngOnDestroy();

    expect(unsubscribeSpy).toHaveBeenCalled();
  });

  it('should emit done onCancel', () => {
    contactsListsService.getContactsList.and.returnValue(of({}));
    editContactsListComponent.ngOnInit();

    const doneSpy = spyOn(editContactsListComponent.done, 'emit');

    editContactsListComponent.onCancel();

    expect(doneSpy).toHaveBeenCalled();
  });
});
