import { HttpClient, HttpErrorResponse } from '@angular/common/http';
import { TestBed } from '@angular/core/testing';
import {
  HttpClientTestingModule,
  HttpTestingController
} from '@angular/common/http/testing';

import { ContactsService } from './contacts.service';

describe('ContactsService', () => {
  let http: HttpClient;
  let httpTestingController: HttpTestingController;
  let contactsService: ContactsService;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule]
    });

    contactsService = TestBed.get(ContactsService);
    http = TestBed.get(HttpClient);
    httpTestingController = TestBed.get(HttpTestingController);
  });

  afterEach(() => {
    httpTestingController.verify();
  });

  describe('getContacts', () => {
    it('should call get with /lists/:listId/contacts', () => {
      const listId = '123';
      const getSpy = spyOn(http, 'get').and.callThrough();

      contactsService.getContacts(listId);

      expect(getSpy).toHaveBeenCalledWith(
        'http://localhost:3000/lists/123/contacts'
      );
    });

    it('should return the server data on success', () => {
      const listId = '123';
      const serverReturn = [{ name: 'foo', id: '456' }];

      contactsService.getContacts(listId).subscribe(data => {
        expect(data).toEqual(serverReturn);
      });

      const req = httpTestingController.expectOne({
        method: 'GET',
        url: 'http://localhost:3000/lists/123/contacts'
      });

      req.flush(serverReturn);
    });

    it('should return an error observable on failure', () => {
      const listId = '123';
      const serverReturn = {};

      contactsService.getContacts(listId).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        e => {
          expect(e instanceof Error).toBe(true);
        }
      );

      const req = httpTestingController.expectOne({
        method: 'GET',
        url: 'http://localhost:3000/lists/123/contacts'
      });

      req.flush(serverReturn, { status: 500, statusText: 'Internal Error' });
    });
  });

  describe('addContact', () => {
    it('should call post with /lists/123/contacts and the given data', () => {
      const listId = '123';
      const testData = { name: 'foo', email: '', tel: '' };
      const postSpy = spyOn(http, 'post').and.callThrough();

      contactsService.addContact(listId, testData);

      expect(postSpy).toHaveBeenCalledWith(
        'http://localhost:3000/lists/123/contacts',
        testData
      );
    });

    it('should return the server data on success', () => {
      const listId = '123';
      const testData = { name: 'foo', email: '', tel: '' };
      const serverReturn = { name: 'foo', email: '', tel: '', id: '123' };

      contactsService.addContact(listId, testData).subscribe(data => {
        expect(data).toEqual(serverReturn);
      });

      const req = httpTestingController.expectOne({
        method: 'POST',
        url: 'http://localhost:3000/lists/123/contacts'
      });

      req.flush(serverReturn);
    });

    it('should return an error observable on failure', () => {
      const listId = '123';
      const testData = { name: 'foo', email: '', tel: '' };
      const serverReturn = {};

      contactsService.addContact(listId, testData).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        e => {
          expect(e instanceof Error).toBe(true);
        }
      );

      const req = httpTestingController.expectOne({
        method: 'POST',
        url: 'http://localhost:3000/lists/123/contacts'
      });

      req.flush(serverReturn, { status: 500, statusText: 'Internal Error' });
    });
  });

  describe('updateContact', () => {
    it('should call put with /lists/123/contacts/456 and the given data', () => {
      const listId = '123';
      const contactId = '456';
      const testData = { name: 'foo', email: '', tel: '' };
      const pustSpy = spyOn(http, 'put').and.callThrough();

      contactsService.updateContact(listId, contactId, testData);

      expect(pustSpy).toHaveBeenCalledWith(
        'http://localhost:3000/lists/123/contacts/456',
        testData
      );
    });

    it('should return no data on success', () => {
      const listId = '123';
      const contactId = '456';
      const testData = { name: 'foo', email: '', tel: '' };
      const serverReturn = {};

      contactsService
        .updateContact(listId, contactId, testData)
        .subscribe(data => {
          expect(data).toBe(serverReturn);
        });

      const req = httpTestingController.expectOne({
        method: 'PUT',
        url: 'http://localhost:3000/lists/123/contacts/456'
      });

      req.flush(serverReturn);
    });

    it('should return an error observable on failure', () => {
      const listId = '123';
      const contactId = '456';
      const testData = { name: 'foo', email: '', tel: '' };
      const serverReturn = {};

      contactsService.updateContact(listId, contactId, testData).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        e => {
          expect(e instanceof Error).toBe(true);
        }
      );

      const req = httpTestingController.expectOne({
        method: 'PUT',
        url: 'http://localhost:3000/lists/123/contacts/456'
      });

      req.flush(serverReturn, { status: 500, statusText: 'Internal Error' });
    });
  });

  describe('getContact', () => {
    it('should call get with /lists/123/contacts/456', () => {
      const listId = '123';
      const contactId = '456';
      const getSpy = spyOn(http, 'get').and.callThrough();

      contactsService.getContact(listId, contactId);

      expect(getSpy).toHaveBeenCalledWith(
        'http://localhost:3000/lists/123/contacts/456'
      );
    });

    it('should return the server data on success', () => {
      const listId = '123';
      const contactId = '456';
      const serverReturn = { id: contactId, name: 'foo', email: '', tel: '' };

      contactsService.getContact(listId, contactId).subscribe(data => {
        expect(data).toBe(serverReturn);
      });

      const req = httpTestingController.expectOne({
        method: 'GET',
        url: 'http://localhost:3000/lists/123/contacts/456'
      });

      req.flush(serverReturn);
    });

    it('should return an error observable on failure', () => {
      const listId = '123';
      const contactId = '456';
      const serverReturn = {};

      contactsService.getContact(listId, contactId).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        e => {
          expect(e instanceof Error).toBe(true);
        }
      );

      const req = httpTestingController.expectOne({
        method: 'GET',
        url: 'http://localhost:3000/lists/123/contacts/456'
      });

      req.flush(serverReturn, { status: 500, statusText: 'Internal Error' });
    });
  });

  describe('deleteContact', () => {
    it('should call delete with /lists/123/contacts/456', () => {
      const listId = '123';
      const contactId = '456';
      const deleteSpy = spyOn(http, 'delete').and.callThrough();

      contactsService.deleteContact(listId, contactId);

      expect(deleteSpy).toHaveBeenCalledWith(
        'http://localhost:3000/lists/123/contacts/456'
      );
    });

    it('should return the server data on success', () => {
      const listId = '123';
      const contactId = '456';
      const serverReturn = {};

      contactsService.deleteContact(listId, contactId).subscribe(data => {
        expect(data).toBe(serverReturn);
      });

      const req = httpTestingController.expectOne({
        method: 'DELETE',
        url: 'http://localhost:3000/lists/123/contacts/456'
      });

      req.flush(serverReturn);
    });

    it('should return an error observable on failure', () => {
      const listId = '123';
      const contactId = '456';
      const serverReturn = {};

      contactsService.deleteContact(listId, contactId).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        e => {
          expect(e instanceof Error).toBe(true);
        }
      );

      const req = httpTestingController.expectOne({
        method: 'DELETE',
        url: 'http://localhost:3000/lists/123/contacts/456'
      });

      req.flush(serverReturn, { status: 500, statusText: 'Internal Error' });
    });
  });

  describe('handleError', () => {
    it(`should return 'Verbindungsfehler' for a ProgressEvent`, () => {
      const error = new HttpErrorResponse({
        error: new ProgressEvent('some event')
      });

      contactsService.handleError(error).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        (err: Error) => {
          expect(err.message).toBe('Verbindungsfehler');
        }
      );
    });

    it(`should return 'Fehlerhafte Daten' for a status 400`, () => {
      const error = new HttpErrorResponse({
        status: 400
      });

      contactsService.handleError(error).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        (err: Error) => {
          expect(err.message).toBe('Fehlerhafte Daten');
        }
      );
    });

    it(`should return 'Nicht gefunden' for a status 404`, () => {
      const error = new HttpErrorResponse({
        status: 404
      });

      contactsService.handleError(error).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        (err: Error) => {
          expect(err.message).toBe('Nicht gefunden');
        }
      );
    });

    it(`should return 'Serverfehler' for a status 500`, () => {
      const error = new HttpErrorResponse({
        status: 500
      });

      contactsService.handleError(error).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        (err: Error) => {
          expect(err.message).toBe('Serverfehler');
        }
      );
    });

    it(`should return 'Unerwarteter Fehler' in other cases`, () => {
      const error = new HttpErrorResponse({
        status: 501
      });

      contactsService.handleError(error).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        (err: Error) => {
          expect(err.message).toBe('Unerwarteter Fehler');
        }
      );
    });
  });
});
