import { ComponentFixture, TestBed } from '@angular/core/testing';
import { NO_ERRORS_SCHEMA } from '@angular/core';
import { By } from '@angular/platform-browser';
import { of, throwError } from 'rxjs';

import { EditContactComponent } from './edit-contact.component';
import { ContactsService } from '../contacts.service';

describe('EditContactComponent', () => {
  let fixture: ComponentFixture<EditContactComponent>;
  let editContactComponent: EditContactComponent;
  let contactsService: jasmine.SpyObj<ContactsService>;

  beforeEach(() => {
    TestBed.configureTestingModule({
      declarations: [EditContactComponent],
      providers: [
        {
          provide: ContactsService,
          useValue: jasmine.createSpyObj('ContactsService', [
            'updateContact',
            'getContact'
          ])
        }
      ],
      schemas: [NO_ERRORS_SCHEMA]
    });

    contactsService = TestBed.get(ContactsService);

    fixture = TestBed.createComponent(EditContactComponent);
    editContactComponent = fixture.componentInstance;
  });

  it('should call getContact on init', () => {
    contactsService.getContact.and.returnValue(of({}));
    const listId = '123';
    const contactId = '456';
    editContactComponent.contactId = contactId;
    editContactComponent.currentContactsListId = listId;

    editContactComponent.ngOnInit();

    expect(contactsService.getContact).toHaveBeenCalledWith(listId, contactId);
  });

  it('should set the contact property if the getContact call was successful', () => {
    const listId = '123';
    const contactId = '456';
    const contact = {
      id: contactId,
      name: 'foobar'
    };
    contactsService.getContact.and.returnValue(of(contact));

    editContactComponent.contactId = contactId;
    editContactComponent.currentContactsListId = listId;

    editContactComponent.ngOnInit();

    expect(editContactComponent.contact).toEqual(contact);
  });

  it('should show an error message if the getContact call failed', () => {
    const msg = 'SOME_ERROR';
    contactsService.getContact.and.returnValue(throwError(new Error(msg)));
    editContactComponent.ngOnInit();

    fixture.detectChanges();

    const elem = fixture.debugElement.query(By.css('.error'));

    expect(elem.nativeElement.textContent).toContain(msg);
  });

  it('should call updateContact on submit', () => {
    contactsService.updateContact.and.returnValue(of(''));
    contactsService.getContact.and.returnValue(of({}));
    editContactComponent.ngOnInit();

    const listId = '123';
    const contactId = '456';
    const data = { name: 'abc' };

    editContactComponent.currentContactsListId = listId;
    editContactComponent.contactId = contactId;

    editContactComponent.onSubmit(data);

    expect(contactsService.updateContact).toHaveBeenCalledWith(
      listId,
      contactId,
      data
    );
  });

  it('should emit done if the updateContact call was successful', () => {
    contactsService.updateContact.and.returnValue(of(''));
    contactsService.getContact.and.returnValue(of({}));
    editContactComponent.ngOnInit();

    const id = '123';
    const data = { name: 'abc' };

    editContactComponent.currentContactsListId = id;

    const doneSpy = spyOn(editContactComponent.done, 'emit');

    editContactComponent.onSubmit(data);

    expect(doneSpy).toHaveBeenCalled();
  });

  it('should show an error message if the updateContact call failed', () => {
    const msg = 'SOME_ERROR';
    contactsService.updateContact.and.returnValue(throwError(new Error(msg)));
    contactsService.getContact.and.returnValue(of({}));
    editContactComponent.ngOnInit();

    const id = '123';
    const data = { name: 'abc' };

    editContactComponent.currentContactsListId = id;

    editContactComponent.onSubmit(data);

    fixture.detectChanges();

    const elem = fixture.debugElement.query(By.css('.error'));

    expect(elem.nativeElement.textContent).toContain(msg);
  });

  it('should unsubscribe the update subscription on destroy', () => {
    const id = '123';
    const data = { name: 'abc' };
    contactsService.getContact.and.returnValue(of({}));
    contactsService.updateContact.and.returnValue(of(''));
    editContactComponent.ngOnInit();

    editContactComponent.currentContactsListId = id;

    editContactComponent.onSubmit(data);

    const unsubscribeSpy = spyOn(
      editContactComponent.updateSubscription,
      'unsubscribe'
    ).and.callThrough();

    editContactComponent.ngOnDestroy();

    expect(unsubscribeSpy).toHaveBeenCalled();
  });

  it('should unsubscribe the get subscription on destroy', () => {
    contactsService.getContact.and.returnValue(of({}));
    editContactComponent.ngOnInit();

    const unsubscribeSpy = spyOn(
      editContactComponent.getSubscription,
      'unsubscribe'
    ).and.callThrough();

    editContactComponent.ngOnDestroy();

    expect(unsubscribeSpy).toHaveBeenCalled();
  });

  it('should emit done onCancel', () => {
    contactsService.getContact.and.returnValue(of({}));
    editContactComponent.ngOnInit();

    const doneSpy = spyOn(editContactComponent.done, 'emit');

    editContactComponent.onCancel();

    expect(doneSpy).toHaveBeenCalled();
  });
});
