import { ComponentFixture, TestBed } from '@angular/core/testing';
import { By } from '@angular/platform-browser';
import { Router } from '@angular/router';
import { FormGroupDirective, ReactiveFormsModule } from '@angular/forms';
import { of, throwError } from 'rxjs';

import { LoginComponent } from './login.component';
import { LoginService } from './login.service';

describe('LoginComponent', () => {
  let fixture: ComponentFixture<LoginComponent>;
  let loginComponent: LoginComponent;
  let loginService: jasmine.SpyObj<LoginService>;
  let router: Router;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [ReactiveFormsModule],
      declarations: [LoginComponent],
      providers: [
        {
          provide: LoginService,
          useValue: jasmine.createSpyObj('LoginService', ['execLogin'])
        },
        {
          provide: Router,
          useValue: jasmine.createSpyObj('Router', ['navigate'])
        }
      ]
    });

    loginService = TestBed.get(LoginService);
    router = TestBed.get(Router);

    fixture = TestBed.createComponent(LoginComponent);
    loginComponent = fixture.componentInstance;
  });

  it('should call execLogin with the form data on submit if the form is valid', () => {
    const data = { username: 'foo', password: 'bar' };

    loginService.execLogin.and.returnValue(of(''));

    // setup form
    loginComponent.ngOnInit();

    loginComponent.myForm.setValue(data);

    loginComponent.onSubmit();

    expect(loginService.execLogin).toHaveBeenCalledWith(data);
  });

  it('should not call execLogin if the form is invalid', () => {
    // setup form
    loginComponent.ngOnInit();

    loginComponent.onSubmit();

    expect(loginService.execLogin).not.toHaveBeenCalled();
  });

  it('should navigate to / on success', () => {
    const data = { username: 'foo', password: 'bar' };

    loginService.execLogin.and.returnValue(of(''));

    // setup form
    loginComponent.ngOnInit();

    loginComponent.myForm.setValue(data);

    loginComponent.onSubmit();

    expect(router.navigate).toHaveBeenCalledWith(['/']);
  });

  it('should show an error message if the username is invalid and the form submitted', () => {
    const data = { username: '', password: 'bar' };

    // setup form
    loginComponent.ngOnInit();

    loginComponent.myForm.setValue(data);

    fixture.detectChanges();

    const form = fixture.debugElement.query(By.directive(FormGroupDirective));
    form.triggerEventHandler('submit', null);

    fixture.detectChanges();

    // Zugriff auf das erste Element mit der form-error Klasse
    const elem = fixture.debugElement.query(By.css('.form-error'));

    expect(elem.nativeElement.textContent).toContain('Pflichtfeld');
  });

  it('should show an error message if the password is invalid and the form submitted', () => {
    const data = { username: 'foo', password: '' };

    // setup form
    loginComponent.ngOnInit();

    loginComponent.myForm.setValue(data);

    fixture.detectChanges();

    const form = fixture.debugElement.query(By.directive(FormGroupDirective));
    form.triggerEventHandler('submit', null);

    fixture.detectChanges();

    // Zugriff auf das zweite Element mit der form-error Klasse
    const elem = fixture.debugElement.queryAll(By.css('.form-error'))[1];

    expect(elem.nativeElement.textContent).toContain('Pflichtfeld');
  });

  it('should show an error message on failure', () => {
    const data = { username: 'foo', password: 'bar' };
    const msg = 'SOME_ERROR';

    loginService.execLogin.and.returnValue(throwError(new Error(msg)));

    // setup form
    loginComponent.ngOnInit();

    loginComponent.myForm.setValue(data);

    loginComponent.onSubmit();

    fixture.detectChanges();

    const elem = fixture.debugElement.query(By.css('.error'));

    expect(elem.nativeElement.textContent).toContain(msg);
  });

  it('should unsubscribe the subscription on destroy', () => {
    const data = { username: 'foo', password: 'bar' };

    loginService.execLogin.and.returnValue(of(''));

    // setup form
    loginComponent.ngOnInit();

    loginComponent.myForm.setValue(data);

    loginComponent.onSubmit();

    const unsubscribeSpy = spyOn(
      loginComponent.subscription,
      'unsubscribe'
    ).and.callThrough();

    loginComponent.ngOnDestroy();

    expect(unsubscribeSpy).toHaveBeenCalled();
  });
});
