import { HttpClient, HttpErrorResponse } from '@angular/common/http';
import { TestBed } from '@angular/core/testing';
import {
  HttpClientTestingModule,
  HttpTestingController
} from '@angular/common/http/testing';

import { ContactsListsService } from './contacts-lists.service';

describe('ContactsListsService', () => {
  let http: HttpClient;
  let httpTestingController: HttpTestingController;
  let contactsListsService: ContactsListsService;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule]
    });

    contactsListsService = TestBed.get(ContactsListsService);
    http = TestBed.get(HttpClient);
    httpTestingController = TestBed.get(HttpTestingController);
  });

  afterEach(() => {
    httpTestingController.verify();
  });

  describe('getContactsLists', () => {
    it('should call get with /lists', () => {
      const getSpy = spyOn(http, 'get').and.callThrough();

      contactsListsService.getContactsLists();

      expect(getSpy).toHaveBeenCalledWith('http://localhost:3000/lists');
    });

    it('should return the server data on success', () => {
      const serverReturn = [{ name: 'foo', id: '456' }];

      contactsListsService.getContactsLists().subscribe(data => {
        expect(data).toEqual(serverReturn);
      });

      const req = httpTestingController.expectOne({
        method: 'GET',
        url: 'http://localhost:3000/lists'
      });

      req.flush(serverReturn);
    });

    it('should return an error observable on failure', () => {
      const serverReturn = {};

      contactsListsService.getContactsLists().subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        e => {
          expect(e instanceof Error).toBe(true);
        }
      );

      const req = httpTestingController.expectOne({
        method: 'GET',
        url: 'http://localhost:3000/lists'
      });

      req.flush(serverReturn, { status: 500, statusText: 'Internal Error' });
    });
  });

  describe('addContactsList', () => {
    it('should call post with /lists and the given data', () => {
      const testData = { name: 'foo' };
      const postSpy = spyOn(http, 'post').and.callThrough();

      contactsListsService.addContactsList(testData);

      expect(postSpy).toHaveBeenCalledWith(
        'http://localhost:3000/lists',
        testData
      );
    });

    it('should return the server data on success', () => {
      const testData = { name: 'foo' };
      const serverReturn = { name: 'foo', id: '123' };

      contactsListsService.addContactsList(testData).subscribe(data => {
        expect(data).toEqual(serverReturn);
      });

      const req = httpTestingController.expectOne({
        method: 'POST',
        url: 'http://localhost:3000/lists'
      });

      req.flush(serverReturn);
    });

    it('should return an error observable on failure', () => {
      const testData = { name: 'foo' };
      const serverReturn = {};

      contactsListsService.addContactsList(testData).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        e => {
          expect(e instanceof Error).toBe(true);
        }
      );

      const req = httpTestingController.expectOne({
        method: 'POST',
        url: 'http://localhost:3000/lists'
      });

      req.flush(serverReturn, { status: 500, statusText: 'Internal Error' });
    });
  });

  describe('updateContactsList', () => {
    it('should call put with /lists/123 and the given data', () => {
      const listId = '123';
      const testData = { name: 'foo' };
      const pustSpy = spyOn(http, 'put').and.callThrough();

      contactsListsService.updateContactsList(listId, testData);

      expect(pustSpy).toHaveBeenCalledWith(
        'http://localhost:3000/lists/123',
        testData
      );
    });

    it('should return no data on success', () => {
      const listId = '123';
      const testData = { name: 'foo' };
      const serverReturn = {};

      contactsListsService
        .updateContactsList(listId, testData)
        .subscribe(data => {
          expect(data).toBe(serverReturn);
        });

      const req = httpTestingController.expectOne({
        method: 'PUT',
        url: 'http://localhost:3000/lists/123'
      });

      req.flush(serverReturn);
    });

    it('should return an error observable on failure', () => {
      const listId = '123';
      const testData = { name: 'foo' };
      const serverReturn = {};

      contactsListsService.updateContactsList(listId, testData).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        e => {
          expect(e instanceof Error).toBe(true);
        }
      );

      const req = httpTestingController.expectOne({
        method: 'PUT',
        url: 'http://localhost:3000/lists/123'
      });

      req.flush(serverReturn, { status: 500, statusText: 'Internal Error' });
    });
  });

  describe('getContactsList', () => {
    it('should call get with /lists/123', () => {
      const listId = '123';
      const getSpy = spyOn(http, 'get').and.callThrough();

      contactsListsService.getContactsList(listId);

      expect(getSpy).toHaveBeenCalledWith('http://localhost:3000/lists/123');
    });

    it('should return the server data on success', () => {
      const listId = '123';
      const serverReturn = { id: listId, name: 'foo' };

      contactsListsService.getContactsList(listId).subscribe(data => {
        expect(data).toBe(serverReturn);
      });

      const req = httpTestingController.expectOne({
        method: 'GET',
        url: 'http://localhost:3000/lists/123'
      });

      req.flush(serverReturn);
    });

    it('should return an error observable on failure', () => {
      const listId = '123';
      const serverReturn = {};

      contactsListsService.getContactsList(listId).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        e => {
          expect(e instanceof Error).toBe(true);
        }
      );

      const req = httpTestingController.expectOne({
        method: 'GET',
        url: 'http://localhost:3000/lists/123'
      });

      req.flush(serverReturn, { status: 500, statusText: 'Internal Error' });
    });
  });

  describe('deleteContactsList', () => {
    it('should call delete with /lists/123/contacts/456', () => {
      const listId = '123';
      const deleteSpy = spyOn(http, 'delete').and.callThrough();

      contactsListsService.deleteContactsList(listId);

      expect(deleteSpy).toHaveBeenCalledWith('http://localhost:3000/lists/123');
    });

    it('should return the server data on success', () => {
      const listId = '123';
      const serverReturn = {};

      contactsListsService.deleteContactsList(listId).subscribe(data => {
        expect(data).toBe(serverReturn);
      });

      const req = httpTestingController.expectOne({
        method: 'DELETE',
        url: 'http://localhost:3000/lists/123'
      });

      req.flush(serverReturn);
    });

    it('should return an error observable on failure', () => {
      const listId = '123';
      const serverReturn = {};

      contactsListsService.deleteContactsList(listId).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        e => {
          expect(e instanceof Error).toBe(true);
        }
      );

      const req = httpTestingController.expectOne({
        method: 'DELETE',
        url: 'http://localhost:3000/lists/123'
      });

      req.flush(serverReturn, { status: 500, statusText: 'Internal Error' });
    });
  });

  describe('handleError', () => {
    it(`should return 'Verbindungsfehler' for a ProgressEvent`, () => {
      const error = new HttpErrorResponse({
        error: new ProgressEvent('some event')
      });

      contactsListsService.handleError(error).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        (err: Error) => {
          expect(err.message).toBe('Verbindungsfehler');
        }
      );
    });

    it(`should return 'Fehlerhafte Daten' for a status 400`, () => {
      const error = new HttpErrorResponse({
        status: 400
      });

      contactsListsService.handleError(error).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        (err: Error) => {
          expect(err.message).toBe('Fehlerhafte Daten');
        }
      );
    });

    it(`should return 'Nicht gefunden' for a status 404`, () => {
      const error = new HttpErrorResponse({
        status: 404
      });

      contactsListsService.handleError(error).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        (err: Error) => {
          expect(err.message).toBe('Nicht gefunden');
        }
      );
    });

    it(`should return 'Serverfehler' for a status 500`, () => {
      const error = new HttpErrorResponse({
        status: 500
      });

      contactsListsService.handleError(error).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        (err: Error) => {
          expect(err.message).toBe('Serverfehler');
        }
      );
    });

    it(`should return 'Unerwarteter Fehler' in other cases`, () => {
      const error = new HttpErrorResponse({
        status: 501
      });

      contactsListsService.handleError(error).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        (err: Error) => {
          expect(err.message).toBe('Unerwarteter Fehler');
        }
      );
    });
  });
});
