import { ComponentFixture, TestBed } from '@angular/core/testing';
import {
  ActivatedRoute,
  convertToParamMap,
  ParamMap,
  Params
} from '@angular/router';
import { NO_ERRORS_SCHEMA, Component } from '@angular/core';
import { of, throwError, ReplaySubject } from 'rxjs';
import { By } from '@angular/platform-browser';

import { ContactsWrapperComponent } from './contacts-wrapper.component';
import { ContactsService } from '../contacts.service';

@Component({
  selector: 'app-contacts',
  template: ''
})
class ContactsStubComponent {}

@Component({
  selector: 'app-add-contact',
  template: ''
})
class AddContactStubComponent {}

@Component({
  selector: 'app-edit-contact',
  template: ''
})
class EditContactStubComponent {}

class ActivatedRouteStub {
  private subject = new ReplaySubject<ParamMap>();

  paramMap = this.subject.asObservable();

  setParamMap(params: Params) {
    this.subject.next(convertToParamMap(params));
  }
}

describe('ContactsWrapperComponent', () => {
  let fixture: ComponentFixture<ContactsWrapperComponent>;
  let contactsWrapperComponent: ContactsWrapperComponent;
  let contactsService: jasmine.SpyObj<ContactsService>;
  let route: ActivatedRouteStub;

  beforeEach(() => {
    TestBed.configureTestingModule({
      declarations: [
        ContactsWrapperComponent,
        ContactsStubComponent,
        AddContactStubComponent,
        EditContactStubComponent
      ],
      providers: [
        {
          provide: ContactsService,
          useValue: jasmine.createSpyObj('ContactsService', [
            'getContacts',
            'deleteContact'
          ])
        },
        { provide: ActivatedRoute, useClass: ActivatedRouteStub }
      ],
      schemas: [NO_ERRORS_SCHEMA]
    });

    contactsService = TestBed.get(ContactsService);
    route = TestBed.get(ActivatedRoute);

    fixture = TestBed.createComponent(ContactsWrapperComponent);
    contactsWrapperComponent = fixture.componentInstance;
  });

  describe('getContacts', () => {
    it('should call the getContacts method of the service', () => {
      contactsService.getContacts.and.returnValue(of([]));

      contactsWrapperComponent.getContacts();

      expect(contactsService.getContacts).toHaveBeenCalled();
    });

    it('should set the value of contacts', () => {
      const serverReturn = [{ name: 'foo', id: '123' }];
      contactsService.getContacts.and.returnValue(of(serverReturn));

      contactsWrapperComponent.getContacts();

      expect(contactsWrapperComponent.contacts).toEqual(serverReturn);
    });

    it('should show an error message if getting the data failed', () => {
      const msg = 'SOME_ERROR';

      contactsService.getContacts.and.returnValue(throwError(new Error(msg)));

      contactsWrapperComponent.getContacts();

      fixture.detectChanges();

      const elem = fixture.debugElement.query(By.css('.error'));

      expect(elem.nativeElement.textContent).toContain(msg);
    });
  });

  it('should get the contacts lists onInit', () => {
    const getContactsListsSpy = spyOn(contactsWrapperComponent, 'getContacts');

    contactsWrapperComponent.ngOnInit();
    route.setParamMap({ listId: '123' });

    expect(getContactsListsSpy).toHaveBeenCalled();
  });

  it('should show the ContactsComponent when initialized', () => {
    contactsService.getContacts.and.returnValue(of([]));
    route.setParamMap({ listId: '123' });

    fixture.detectChanges();

    expect(contactsWrapperComponent.display).toBe('list');

    const elem = fixture.debugElement.query(
      By.directive(ContactsStubComponent)
    );

    expect(elem).not.toBeNull();
  });

  it('should display AddContact onAdd', () => {
    contactsService.getContacts.and.returnValue(of([]));
    route.setParamMap({ listId: '123' });

    contactsWrapperComponent.onAdd();

    fixture.detectChanges();

    expect(contactsWrapperComponent.display).toBe('add');

    const elem = fixture.debugElement.query(
      By.directive(AddContactStubComponent)
    );

    expect(elem).not.toBeNull();
  });

  it('should display ContactsComponent and get contacts onDone', () => {
    const getContactsListsSpy = spyOn(contactsWrapperComponent, 'getContacts');
    route.setParamMap({ listId: '123' });

    contactsWrapperComponent.onDone();

    fixture.detectChanges();

    expect(getContactsListsSpy).toHaveBeenCalled();
    expect(contactsWrapperComponent.display).toBe('list');

    const elem = fixture.debugElement.query(
      By.directive(ContactsStubComponent)
    );

    expect(elem).not.toBeNull();
  });

  it('should display EditContact onEdit', () => {
    contactsService.getContacts.and.returnValue(of([]));
    route.setParamMap({ listId: '123' });
    const contactId = '123';
    contactsWrapperComponent.onEdit(contactId);

    fixture.detectChanges();

    expect(contactsWrapperComponent.display).toBe('edit');
    expect(contactsWrapperComponent.currentContactsListId).toBe(contactId);

    const elem = fixture.debugElement.query(
      By.directive(EditContactStubComponent)
    );

    expect(elem).not.toBeNull();
  });

  describe('onDelete', () => {
    it('should call deleteContact with the correct id', () => {
      const listId = '123';
      contactsService.getContacts.and.returnValue(of([]));

      route.setParamMap({ listId: listId });
      contactsWrapperComponent.ngOnInit();

      contactsService.deleteContact.and.returnValue(of(''));

      const contactId = '456';
      contactsWrapperComponent.onDelete(contactId);

      expect(contactsService.deleteContact).toHaveBeenCalledWith(
        listId,
        contactId
      );
    });

    it('should get the contacts lists', () => {
      contactsService.deleteContact.and.returnValue(of(''));

      const getContactsListsSpy = spyOn(
        contactsWrapperComponent,
        'getContacts'
      );

      const contactId = '123';
      contactsWrapperComponent.onDelete(contactId);

      expect(getContactsListsSpy).toHaveBeenCalled();
    });

    it('should show an error message if the deletion failed', () => {
      contactsService.getContacts.and.returnValue(of([]));

      const msg = 'SOME_ERROR';

      contactsService.deleteContact.and.returnValue(throwError(new Error(msg)));

      const contactId = '123';
      contactsWrapperComponent.onDelete(contactId);

      fixture.detectChanges();

      const elem = fixture.debugElement.query(By.css('.error'));

      expect(elem.nativeElement.textContent).toContain(msg);
    });
  });
});
