import { TestBed } from '@angular/core/testing';
import { HttpClient, HttpErrorResponse } from '@angular/common/http';
import {
  HttpClientTestingModule,
  HttpTestingController
} from '@angular/common/http/testing';

import { LoginService } from './login.service';

describe('LoginService', () => {
  let http: HttpClient;
  let httpTestingController: HttpTestingController;
  let loginService: LoginService;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule]
    });

    loginService = TestBed.get(LoginService);
    http = TestBed.get(HttpClient);
    httpTestingController = TestBed.get(HttpTestingController);
  });

  afterEach(() => {
    httpTestingController.verify();
  });

  describe('execLogin', () => {
    it('should call /login with username and password', () => {
      const testData = { username: 'foo', password: 'bar' };
      const postSpy = spyOn(http, 'post').and.callThrough();

      loginService.execLogin(testData);

      expect(postSpy).toHaveBeenCalledWith(
        'http://localhost:3000/login',
        testData,
        { withCredentials: true }
      );
    });

    it('should return the server data on success', () => {
      const testData = { username: 'foo', password: 'bar' };
      const serverReturn = {};

      loginService.execLogin(testData).subscribe(data => {
        expect(data).toEqual(serverReturn);
      });

      const req = httpTestingController.expectOne({
        method: 'POST',
        url: 'http://localhost:3000/login'
      });

      req.flush(serverReturn);
    });

    it('should return an error on failure', () => {
      const testData = { username: 'foo', password: 'bar' };
      const serverReturn = {};

      loginService.execLogin(testData).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        e => {
          expect(e instanceof Error).toBe(true);
        }
      );

      const req = httpTestingController.expectOne({
        method: 'POST',
        url: 'http://localhost:3000/login'
      });

      req.flush(serverReturn, { status: 500, statusText: 'Internal Error' });
    });
  });

  describe('isLoggedIn', () => {
    it('should return true if the server call is successful', () => {
      const serverReturn = {};

      loginService.isLoggedIn().subscribe(data => {
        expect(data).toBe(true);
      });

      const req = httpTestingController.expectOne({
        method: 'GET',
        url: 'http://localhost:3000/login'
      });

      req.flush(serverReturn);
    });

    it('should return false if the server call fails', () => {
      const serverReturn = {};

      loginService.isLoggedIn().subscribe(data => {
        expect(data).toBe(false);
      });

      const req = httpTestingController.expectOne({
        method: 'GET',
        url: 'http://localhost:3000/login'
      });

      req.flush(serverReturn, { status: 401, statusText: 'Unauthorized' });
    });
  });

  describe('handleError', () => {
    it(`should return 'Verbindungsfehler' for a ProgressEvent`, () => {
      const error = new HttpErrorResponse({
        error: new ProgressEvent('some event')
      });

      loginService.handleError(error).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        (err: Error) => {
          expect(err.message).toBe('Verbindungsfehler');
        }
      );
    });

    it(`should return 'Fehlerhafte Daten' for a status 400`, () => {
      const error = new HttpErrorResponse({
        status: 400
      });

      loginService.handleError(error).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        (err: Error) => {
          expect(err.message).toBe('Fehlerhafte Daten');
        }
      );
    });

    it(`should return 'Passwort ist falsch' for a status 401`, () => {
      const error = new HttpErrorResponse({
        status: 401
      });

      loginService.handleError(error).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        (err: Error) => {
          expect(err.message).toBe('Passwort ist falsch');
        }
      );
    });

    it(`should return 'Benutzer nicht gefunden' for a status 404`, () => {
      const error = new HttpErrorResponse({
        status: 404
      });

      loginService.handleError(error).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        (err: Error) => {
          expect(err.message).toBe('Benutzer nicht gefunden');
        }
      );
    });

    it(`should return 'Serverfehler' for a status 500`, () => {
      const error = new HttpErrorResponse({
        status: 500
      });

      loginService.handleError(error).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        (err: Error) => {
          expect(err.message).toBe('Serverfehler');
        }
      );
    });

    it(`should return 'Unerwarteter Fehler' in other cases`, () => {
      const error = new HttpErrorResponse({
        status: 501
      });

      loginService.handleError(error).subscribe(
        () => {
          // Stellt sicher, dass der next-Parameter nicht aufgerufen worden ist
          fail('Expected Error');
        },
        (err: Error) => {
          expect(err.message).toBe('Unerwarteter Fehler');
        }
      );
    });
  });
});
